import json

from django import forms
from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponse
from django.urls import path

from .funcs import indo_upgrade_widget


class IndoResponseMixin:
    def get_indo_urls(self):
        info = self.model._meta.app_label, self.model._meta.model_name
        urls = [
            path(
                '<path:object_id>/indo/',
                self.indo_response,
                name='%s_%s_indo' % info,
            )
        ]
        return urls

    def get_urls(self):
        urls = super().get_urls()
        indo_urls = self.get_indo_urls()
        return indo_urls + urls

    def indo_response_field_value(self, request, obj, field):
        return None

    def indo_make_response(self, request, obj, field):
        # First read_from indo_response_field_value
        value = self.indo_response_field_value(request, obj, field)
        if value is not None:
            return {field: value}

        # Second Check obj has parameter
        if hasattr(obj, field):
            return {field: getattr(obj, field)}

        # third check if callable exists
        admin_field = getattr(self, field, None)
        if callable(admin_field):
            return {field: admin_field(obj)}

        return {}

    def indo_response(self, request, object_id):
        parameter = request.GET.get('indo_field', None)
        if not parameter or parameter == 'pk':
            parameter = self.model._meta.pk.name

        content = {}
        obj = self.get_object(request, object_id)

        if obj:
            content = self.indo_make_response(request, obj, parameter)

        return HttpResponse(
            json.dumps(content, cls=DjangoJSONEncoder),
            content_type="application/json"
        )


class IndoSetFieldMixin:
    indo_fields = {}  # name: fk

    def formfield_for_dbfield(self, db_field, request, **kwargs):
        form_field = super().formfield_for_dbfield(db_field, request, **kwargs)
        if db_field.name in self.indo_fields:
            indo_field = self.indo_fields[db_field.name]

            from_field_name, retrive_field_name = \
                indo_field.split('__', maxsplit=1)

            indo_upgrade_widget(
                form_field.widget,
                db_field.model,
                db_field.name,
                from_field_name,
                retrive_field_name,
                self.admin_site
            )
        return form_field

    def get_indo_media(self):
        js = [
            'indo/indo.js',
        ]
        css = {
            '': ('indo/indo.css', ),
        }
        return forms.Media(js=js, css=css)

    @property
    def media(self):
        super_media = super().media
        return super_media + self.get_indo_media()


class IndoFullMixin(IndoResponseMixin, IndoSetFieldMixin):
    pass



from django.db.models.fields.related import ForeignKey
from django.urls import reverse


def indo_render_attrs(attrs):
    res = ''
    for key, value in attrs.items():
        res += f'{key}={value} '

    return res


def indo_get_url_template(model, field_name, admin_site):
    db_field = model._meta.get_field(field_name)

    site_name = admin_site.name

    if isinstance(db_field, ForeignKey):
        model = db_field.remote_field.model

    model_name = model._meta.model_name
    app_label = model._meta.app_label

    url_template = reverse(
        f'{site_name}:{app_label}_{model_name}_indo',
        kwargs={'object_id': '__pk__'}
    )

    return url_template


def indo_get_attrs(to_field, from_field, retrive_field, url_template, initial=None, set_id=True):
    indo_attrs = {
        'indo': 'true',
        'indo_from_field': from_field,
        'indo_to_field': to_field,
        'indo_url_template': url_template,
        'indo_retrive_field': retrive_field or 'pk'
    }
    if initial:
        indo_attrs['indo_initial'] = 'true'

    if set_id:
        indo_attrs['id'] = f'id_{to_field}'

    return indo_attrs


def indo_upgrade_widget(widget, model, to_field, from_field, retrive_field, admin_site, initial=None, set_id=False):
    indo_attrs = indo_get_attrs(
        to_field,
        from_field,
        retrive_field,
        indo_get_url_template(model, from_field, admin_site),
        initial,
        set_id
    )

    widget.attrs.update(indo_attrs)
    return widget



import os

from setuptools import find_packages, setup


def read(fname):
    path = os.path.join(os.path.dirname(__file__), fname)
    try:
        file = open(path, encoding='utf-8')
    except TypeError:
        file = open(path)
    return file.read()


setup(
    name='django-indo',
    version=__import__('indo').__version__,
    description='Ajax call in Django admin interface',
    long_description=read('README.rst'),
    author='Mohammad Faraji',
    author_email='farajim25@gmail.com',
    url='https://github.com/farajim25/django-indo',
    packages=find_packages(),
    license='MIT',
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Environment :: Web Environment',
        'Topic :: Software Development',
        'Topic :: Software Development :: User Interfaces',
    ],
    install_requires=['Django']
)
